/*
 * Copyright (c) ROSA Laboratory 2011
 * License: GPLv3
 *
 * Authors:
 * Alexey Yermakov <alexey.yermakov@rosalab.ru>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "groupgridwidget.h"

#include <QGraphicsSceneResizeEvent>
#include <QFont>

#include <KDebug>

GroupGridWidget::GroupGridWidget(QGraphicsItem *parent, Qt::WindowFlags wFlags)
  : QGraphicsWidget(parent, wFlags)
{
  m_layout = new QGraphicsLinearLayout(Qt::Vertical);

  setLayout(m_layout);

  m_layout->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
}

GroupGridWidget::~GroupGridWidget()
{
}

GroupGridWidgetGroup* GroupGridWidget::newGroup(QString name)
{
  GroupGridWidgetGroup *group = new GroupGridWidgetGroup();
  m_groupNames.append(name);
  m_groups.append(group);

  m_layout->addItem(group);

  group->setName(name);

  return group;
}

GroupGridWidgetGroup* GroupGridWidget::getGroup(QString name)
{
  if(m_groupNames.contains(name))
    {
      return m_groups[m_groupNames.indexOf(name)];
    }
  else
    {
      return static_cast<GroupGridWidgetGroup*>(NULL);
    };
};
 
void GroupGridWidget::removeGroup(QString name)
{
  int index = m_groupNames.indexOf(name);

  GroupGridWidgetGroup* group = m_groups[index];

  m_layout->removeItem(group);
  group->deleteLater();
  m_groups.removeAt(index);
  m_groupNames.removeAt(index);
}

void GroupGridWidget::clear(void)
{
  while(m_groups.size() > 0)
    {
      GroupGridWidgetGroup* group = m_groups[0];

      m_layout->removeItem(group);
      group->deleteLater();
      m_groups.removeAt(0);
      m_groupNames.removeAt(0);
    }
}

void GroupGridWidget::resizeEvent(QGraphicsSceneResizeEvent *event)
{
  //kDebug() << "Resize event, from " << event->oldSize() << " to " << event->newSize();

  for(int i = 0; i < m_groups.size(); i++)
    {
      m_groups[i]->setMarginsForSize(event->newSize());
    }
}

// GroupGridWidgetGroup

GroupGridWidgetGroup::GroupGridWidgetGroup(QGraphicsItem *parent, Qt::WindowFlags wFlags)
  : QGraphicsWidget(parent, wFlags)
{
  m_layout = new QGraphicsLinearLayout(Qt::Vertical);

  setLayout(m_layout);

  m_nameLabel = new Plasma::Label();
  //m_nameLabel->setStyleSheet("font: bold 16px");
  QFont labelFont;
  labelFont.setPointSize(11);
  labelFont.setBold(true);
  m_nameLabel->setFont(labelFont);
  
  m_layout->addItem(m_nameLabel);

  m_separator = new Plasma::Separator();
  m_layout->addItem(m_separator);

  setAcceptHoverEvents(true);
  m_hoverIndicator = new Plasma::ItemBackground(this);
  m_hoverIndicator->setZValue(-100);
  m_hoverIndicator->hide();
}

GroupGridWidgetGroup::~GroupGridWidgetGroup()
{
  if(m_layout)
    while(m_layout->count())
      {
        m_layout->removeAt(0);
      }
  
  clear();

  // Removing margin objects
  while(m_innerMargins.count())
    {
      QGraphicsWidget *margin = m_innerMargins.at(0);
      m_innerMargins.removeAt(0);

      delete margin;
    }
  
  while(m_outerMargins.count())
    {
      QGraphicsWidget *margin = m_outerMargins.at(0);
      m_outerMargins.removeAt(0);

      delete margin;
    }
  
  delete m_nameLabel;
  delete m_hoverIndicator;
  delete m_separator;
}

void GroupGridWidgetGroup::append(QGraphicsWidget* element)
{
  m_elements.append(element);

  connect(element, SIGNAL(hoverEnter(QGraphicsWidget*)), this, SLOT(setTargetItem(QGraphicsWidget*)));
};

void GroupGridWidgetGroup::clear(void)
{
  for(int i = 0; i < m_elements.size(); i++)
    {
      delete m_elements[i];
    };
  
  m_elements.clear();
};

void GroupGridWidgetGroup::setTargetItem(QGraphicsWidget* item)
{
  m_hoverIndicator->setTargetItem(item);
}

void GroupGridWidgetGroup::hoverLeaveEvent(QGraphicsSceneHoverEvent *evt)
{
  m_hoverIndicator->setTargetItem(NULL);
}

void GroupGridWidgetGroup::repopulate(void)
{
  //kDebug() << "REPOPULATING";
  //kDebug() << "Layout count: " << m_layout->count();
  //kDebug() << "Elements count: " << m_elements.size();

  QGraphicsLinearLayout *marginLayout;
  QGraphicsLinearLayout *marginContentLayout;

  if(m_layout->count() > 2) // Checking if main layout is populated
    {
      //kDebug() << "Getting marginLayout";
      marginLayout = static_cast<QGraphicsLinearLayout*>(m_layout->itemAt(2));
      //kDebug() << "Getting marginContentLayout -- " << marginLayout->count();
      marginContentLayout = static_cast<QGraphicsLinearLayout*>(marginLayout->itemAt(1));
      //kDebug() << "Done";

      //kDebug() << marginContentLayout->count();
      
      while(marginContentLayout->count() > 0)
        {
          marginContentLayout->removeAt(0); // Removing rows
        }

      //kDebug() << "Done2";

      marginLayout->removeAt(0); // Removing marginContentLayout
      m_layout->removeAt(2); // Removing marginLayout

      //kDebug() << "Done3";
    }

  //kDebug() << "Done4";

  // Removing margin objects
  while(m_innerMargins.count())
    {
      QGraphicsWidget *margin = m_innerMargins.at(0);
      m_innerMargins.removeAt(0);

      delete margin;
    }
  
  while(m_outerMargins.count())
    {
      QGraphicsWidget *margin = m_outerMargins.at(0);
      m_outerMargins.removeAt(0);

      delete margin;
    }
  
  // Layout for horizontal margins
  marginLayout = new QGraphicsLinearLayout(Qt::Horizontal);
  marginContentLayout = new QGraphicsLinearLayout(Qt::Vertical);

  // Left margin widget
  QGraphicsWidget *leftMarginWidget = new QGraphicsWidget();
  m_outerMargins.append(leftMarginWidget);
  
  // Right margin widget
  QGraphicsWidget *rightMarginWidget = new QGraphicsWidget();
  m_outerMargins.append(rightMarginWidget);
  
  marginLayout->addItem(leftMarginWidget);
  marginLayout->addItem(marginContentLayout);
  marginLayout->addItem(rightMarginWidget);

  m_layout->addItem(marginLayout);

  QGraphicsLinearLayout *row = NULL;
  for(int i = 0; i < m_elements.size(); i++)
    {
      //kDebug() << "i: " << i;
      //kDebug() << "i mod " << ICONS_IN_A_ROW << ": " << i % ICONS_IN_A_ROW;
      if(i % ICONS_IN_A_ROW == 0)
        {
          row = new QGraphicsLinearLayout(Qt::Horizontal);
          marginContentLayout->addItem(row);
          row->setContentsMargins(0, 0, 0, 0);
        }
      else
        {
          // inter icon margin widget
          QGraphicsWidget *marginWidget = new QGraphicsWidget();
          m_innerMargins.append(marginWidget);
          //kDebug() << marginWidget->minimumSize();
          row->addItem(marginWidget);
        }
      //kDebug() << "Adding element " << i;
      row->addItem(m_elements[i]);
    }

  int iconsInLastRow = m_elements.size() % ICONS_IN_A_ROW;
  int fillersNeeded = iconsInLastRow != 0 ? ICONS_IN_A_ROW - iconsInLastRow : 0;

  //kDebug() << name() << "FillersNeeded: " << fillersNeeded;
  

  // Fillers
  for(int i = 0; i < fillersNeeded; i++)
    {
      // inter icon margin widget
      //kDebug() << "Adding margin";
      QGraphicsWidget *marginWidget = new QGraphicsWidget();
      m_innerMargins.append(marginWidget);
      //kDebug() << marginWidget->minimumSize();
      row->addItem(marginWidget);

      // Filler
      //kDebug() << "Adding filler";
      QGraphicsWidget *fillerWidget = new QGraphicsWidget();
      //fillerWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
      fillerWidget->setMinimumWidth(ICON_SIZE + BUTTON_MARGIN * 2 + BUTTON_MARGIN * 2);
      fillerWidget->setMaximumWidth(ICON_SIZE + BUTTON_MARGIN * 2 + BUTTON_MARGIN * 2);
      row->addItem(fillerWidget);
    }
}

void GroupGridWidgetGroup::setMarginsForSize(QSizeF newSize)
{
  //kDebug() << name() << ": got margins resize request for size - " << newSize;

  int outerMarginSize = INITIAL_OUTERMARGIN_WIDTH;
  
  if(newSize.width() > WIDTH_BORDERLINE + INITIAL_OUTERMARGIN_WIDTH * 2)
    {
      outerMarginSize = (newSize.width() - WIDTH_BORDERLINE) / 2;
    }

  //kDebug() << newSize.width() << " " << WIDTH_BORDERLINE << " " << outerMarginSize;
  
  for(int i = 0; i < m_outerMargins.size(); i++)
    {
      //m_outerMargins[i]->setMinimumWidth(outerMarginSize);
      m_outerMargins[i]->setMaximumWidth(outerMarginSize);
      m_outerMargins[i]->setPreferredWidth(outerMarginSize);
    }

  for(int i = 0; i < m_innerMargins.size(); i++)
    {
      m_innerMargins[i]->setMinimumWidth(0);
    }
  
}
